singular.lib('primdec.lib')
singular.lib('sing.lib')

R = singular.ring(0, '(x,y,z,a1,a2,a3,a4,a6,z1,e0,e1,e2)', 'dp')
x, y, z, a1, a2, a3, a4, a6, z1, e0, e1, e2 = var('x,y,z,a1,a2,a3,a4,a6,z1,e0,e1,e2')

def displaySingularLocus(ideal):
    sL = singular.slocus(ideal.std()).std().minAssGTZ()
    for i in range(1, len(sL) + 1):
        sL[i] = sL[i].std()
    print str(sL) + "\n"

def myAnalysis(n1,n2,n3,n4,n6,gaugeGroup = "",output = 1):
    """
    Performs the Blow-up, computes the residual singular locus and returns the proper transform.

    n1,...,n6: vanishing orders of a1,...,a6 in the Tate model

    gaugeGroup: "SU(2)" or "SU(3)" or "". Is needed to perform the correct blow-up

    output: controls whether any print commands display intermediate steps
    """
    if output:
        print "-------------------------------------------------------------"
        print "Analysing model with ("+str(n1)+","+str(n2)+","+str(n3)+","+str(n4)+","+str(n6)+")"
        print "-------------------------------------------------------------"

    Tate = y^2 + x*y*z*a1 + y*z^3*a3 - (x^3 + x^2*z^2*a2 + x*z^4*a4 + z^6*a6)
    p = Tate.substitute(a1=z1^n1*a1).substitute(a2=z1^n2*a2).substitute(a3=z1^n3*a3).substitute(a4=z1^n4*a4).substitute(a6=z1^n6*a6)

    if output:
        print "Singular locus of Tate form:"
        displaySingularLocus(singular.ideal(str(p)))

    if(gaugeGroup == "SU(2)"):
        p1 = p.substitute(x=e1*x).substitute(y=e1*y).substitute(z1=e0*e1)
        if output: print "A possible SRI is: z*e1, x*y*z, x*y*e0."
    elif (gaugeGroup == "SU(3)"):
        p1 = p.substitute(x=e1*e2*x).substitute(y=e1*e2^2*y).substitute(z1=e0*e1*e2)
        if output: print "A possible SRI is: y*e1, z*e1, z*e2, x*y*z, x*y*e0, x*e0*e2."
    else:
        print "The Tate form is given by: " + str(p)
        return True #This is the end of the analysis since a resolution is not possible.

    # Remove exceptional divisors from equation
    PT = singular.ideal(str(p1)).std().minAssGTZ()[2].std()
    if output:
        print "\nProper transform:\n" + str(PT)

        print '\nSingular Locus of proper transform:'
        displaySingularLocus(PT)

        print "\n\n"
    return PT

def analyseCodim1(PT,gaugeGroup = ""):
    print "ANALYSIS OF CODIMENSION-ONE LOCUS"
    if(gaugeGroup == "SU(2)"):
        E0 = (singular.ideal('e0').std() + PT).std()
        E1 = (singular.ideal('e1').std() + PT).std()
        print 'E0:\n' + str(E0.minAssGTZ())
        print 'E1:\n' + str(E1.minAssGTZ())
    elif(gaugeGroup == "SU(3)"):
        E0 = (singular.ideal('e0').std() + PT).std()
        E1 = (singular.ideal('e1').std() + PT).std()
        E2 = (singular.ideal('e2').std() + PT).std()
        print 'E0:\n' + str(E0.minAssGTZ())
        print 'E1:\n' + str(E1.minAssGTZ())
        print 'E2:\n' + str(E2.minAssGTZ())
    else:
        print "ERROR"
        return False
    print "\n"

def analyseCodim2(PT,gaugeGroup = ""):
    print "ANALYSIS OF CODIMENSION-TWO LOCUS"
    if(gaugeGroup == "SU(2)"):
        print "Analyse a4 --> 0."
        E0 = (singular.ideal('e0').std() + PT + singular.ideal('a4').std()).std()
        E1 = (singular.ideal('e1').std() + PT + singular.ideal('a4').std()).std()
        print 'E0:\n' + str(E0.minAssGTZ())
        print 'E1:\n' + str(E1.minAssGTZ())
    elif(gaugeGroup == "SU(3)"):
        print "Analyse a3 --> 0."
        E0 = (singular.ideal('e0').std() + PT + singular.ideal('a3').std()).std()
        E1 = (singular.ideal('e1').std() + PT + singular.ideal('a3').std()).std()
        E2 = (singular.ideal('e2').std() + PT + singular.ideal('a3').std()).std()
        print 'E0:\n' + str(E0.minAssGTZ())
        print 'E1:\n' + str(E1.minAssGTZ())
        print 'E2:\n' + str(E2.minAssGTZ())
    else:
        print "ERROR"
        return False
    print "\n"


PT12 = myAnalysis(1,1,1,1,2,"SU(2)",0)
PT13 = myAnalysis(1,2,2,1,3,"SU(2)",0)
PT14 = myAnalysis(2,3,2,1,4,"SU(2)",0)
PT15 = myAnalysis(2,4,3,1,5,"SU(2)",0)
PT16 = myAnalysis(3,5,3,1,6,"SU(2)",0)
PT17 = myAnalysis(3,6,4,1,7,"SU(2)",0)
PT22 = myAnalysis(1,1,1,2,3,"SU(3)",0)

analyseCodim1(PT12,"SU(2)")
analyseCodim1(PT13,"SU(2)")
analyseCodim1(PT14,"SU(2)")
analyseCodim1(PT15,"SU(2)")
analyseCodim1(PT16,"SU(2)")
analyseCodim1(PT17,"SU(2)")
analyseCodim1(PT22,"SU(3)")

analyseCodim2(PT12,"SU(2)")
analyseCodim2(PT13,"SU(2)")
analyseCodim2(PT14,"SU(2)")
analyseCodim2(PT15,"SU(2)")
analyseCodim2(PT16,"SU(2)")
analyseCodim2(PT17,"SU(2)")
analyseCodim2(PT22,"SU(3)")